using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Microsoft.Win32;

namespace ThumbViewer
{
    public partial class MainForm : Form
    {
        private int m_nMargin = 5;
        private bool m_bOpenFile = false;
        private String m_strFile = "";

        public MainForm(String strFile)
        {
            m_strFile = strFile;
            InitializeComponent();
        }

        private void OnLoad(object sender, EventArgs e)
        {
            if(!m_strFile.Equals(String.Empty))
                OpenFile(m_strFile);
            
            GetSettings();
        }

        private void OnResize(object sender, EventArgs e)
        {
            // move any thumbnails already present
            int nHorizontal = -1;
            int nControls = this.panelPhotoThumbs.Controls.Count;
            if (nControls > 0)
            {
                for (int i = 0; i < nControls; i++)
                {
                    ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
                    if (nHorizontal < 0)
                    {
                        // determine how many thumbnails can be displayed on one row
                        nHorizontal = (int)(this.panelPhotoThumbs.Width / (thumbnail.Width + 2 * m_nMargin));
                    }
                    if (thumbnail != null)
                    {
                        // set the position for the thumbnail and add it to the panel's controls
                        thumbnail.Left = m_nMargin + (thumbnail.Width + m_nMargin) * (i % nHorizontal);
                        thumbnail.Top = m_nMargin + (thumbnail.Height + m_nMargin) * (i / nHorizontal);
                    }
                }

                this.panelPhotoThumbs.Invalidate();
            }
        }

        private void OnDragEnter(object sender, DragEventArgs e)
        {
            // If the data is a file or a bitmap, display the copy cursor.
            if (e.Data.GetDataPresent(DataFormats.Bitmap) ||
               e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        private void OnDragDrop(object sender, DragEventArgs e)
        {
            // Handle FileDrop data.
            if (e.Data.GetDataPresent(DataFormats.FileDrop))
            {
                // Assign the file names to a string array, in 
                // case the user has selected multiple files.
                String[] files = (String[])e.Data.GetData(DataFormats.FileDrop);
                try
                {
                    OpenFile(files[0]);
                }
                catch (Exception)
                {
                    MessageBox.Show("It seems selected file is not valid", "Thumbnail Database Viewer", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    return;
                }
            }
        }

        private void GetSettings()
        {
            RegistryKey rkUser = Registry.CurrentUser;
            RegistryKey rkOptions = rkUser.OpenSubKey("Software\\ITSoftware\\ThumbViewer");
            if (rkOptions != null)
            {
                Int32 nTopMost = (Int32)rkOptions.GetValue("TopMost", 1);
                if (nTopMost > 0) this.TopMost = true;
                else this.TopMost = false;

                Int32 nOpenFile = (Int32)rkOptions.GetValue("OpenFile", 1);
                if (nOpenFile > 0) this.m_bOpenFile = true;
                else this.m_bOpenFile = false;
            }
            
            int nControls = this.panelPhotoThumbs.Controls.Count;
            if (nControls > 0)
            {
                for (int i = 0; i < nControls; i++)
                {
                    ThumbNail thumbnail = this.panelPhotoThumbs.Controls[i] as ThumbNail;
                    if (thumbnail != null)
                    {
                        thumbnail.m_bOpenFile = this.m_bOpenFile;
                        if (this.m_bOpenFile)
                            thumbnail.pictureBox.Cursor = Cursors.Hand;
                        else
                            thumbnail.pictureBox.Cursor = Cursors.Arrow;
                    }
                }

                this.panelPhotoThumbs.Invalidate();
            }

        }

        private void OpenFile(String strThumbFile)
        {
            ThumbDB db = new ThumbDB(strThumbFile);

            if (db != null)
            {
                this.panelPhotoThumbs.BackgroundImage = null;
                this.panelPhotoThumbs.Controls.Clear();

                String[] strFiles = db.GetThumbfiles();

                int nHorizontal = -1;
                // create the thumbnails for the selected files
                foreach (String strFile in strFiles)
                {
                    if (strFile.Equals(String.Empty))
                        continue;

                    ThumbNail thumbnail = null;

                    Image pImage = null;
                    try
                    {
                        pImage = db.GetThumbnailImage(strFile);
                    }
                    catch (System.OutOfMemoryException)
                    {
                        pImage = null;
                    }

                    Size xyImage = Size.Empty;
                    try
                    {
                        xyImage = db.GetThumbnailSize(strFile);
                    }
                    catch (System.OutOfMemoryException)
                    {
                        xyImage = Size.Empty;
                    }

                    try
                    {
                        thumbnail = new ThumbNail(strFile, pImage, xyImage);
                    }
                    catch (System.OutOfMemoryException)
                    {
                        thumbnail = null;
                        continue;
                    }

                    if (nHorizontal < 0)
                    {
                        // determine how many thumbnails can be displayed on one row
                        nHorizontal = (int)(this.panelPhotoThumbs.Width / (thumbnail.Width + 2 * m_nMargin));
                    }

                    strThumbFile = strThumbFile.Substring(0,strThumbFile.LastIndexOf("\\")+1);
                    thumbnail.m_strFilePath = strThumbFile;
                    // set the position for the thumbnail and add it to the panel's controls
                    thumbnail.Left = m_nMargin + (thumbnail.Width + m_nMargin) * (this.panelPhotoThumbs.Controls.Count % nHorizontal);
                    thumbnail.Top = m_nMargin + (thumbnail.Height + m_nMargin) * (this.panelPhotoThumbs.Controls.Count / nHorizontal);
                    this.panelPhotoThumbs.Controls.Add(thumbnail);
                }
            }
        }

        private void OnButtonAbout(object sender, EventArgs e)
        {
            AboutForm formAbout = new AboutForm();
            formAbout.ShowDialog(this);
        }

        private void OnButtonOptions(object sender, EventArgs e)
        {
            OptionsForm formOptions = new OptionsForm();
            if (formOptions.ShowDialog(this).Equals(DialogResult.OK))
                GetSettings();
        }

        private void OnButtonSearch(object sender, EventArgs e)
        {
            SearchForm sf = new SearchForm();
            if (sf.ShowDialog(this).Equals(DialogResult.OK))
            {
                String strPath = sf.m_strPath;
                strPath = strPath.TrimStart();
                strPath = strPath.TrimEnd();
                if (!strPath.Equals(String.Empty) && File.Exists(strPath))
                    OpenFile(strPath);
            }
        }

        private void OnButtonClose(object sender, EventArgs e)
        {
            this.Close();
        }
    }
}